﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.OleDb;
using System.Data;

//Progetto realizzato da Ing. Maltraversi Marco: maltra@tin.it  http://www.futuresoftware.it e  http://www.ingegneridelweb.com
// Il contenuto di questo progetto  è pubblicato sotto una Licenza Creative Commons : http://creativecommons.org/licenses/by-nc-sa/2.5/it/


namespace CL_Adapter
{
    public class Adapter
    {
        //ATTRIBUTI
        //Stringa di connessione, specifica che il database si trova nella cartella App_Data 
        private static string connectionString = @"Provider=SQLOLEDB;Data Source=.\\SQLEXPRESS;Integrated Security=SSPI;AttachDbFilename=|DataDirectory|DatabaseEsempio.mdf;";

        //Stringhe Query
        private static string sql_GetFullData = "SELECT ID, Nome, Cognome, Indirizzo, Cap, Citta, Tipologia FROM CLIENTI";
        private static string sql_GetDataDetail = "SELECT ID, Nome, Cognome FROM CLIENTI";
        private static string sql_GetDataByID = sql_GetDataDetail + " WHERE (ID = ?) "; //Seleziona i dati in base all'id

        // connection: la connessione al database
        private OleDbConnection dbConn;
 
        //////////////////////////////////////////////////////////

        // COSTRUTTORE
        public Adapter()
        { 
            this.OpenConnectionToDB();
        } // end costruttore

        //////////////////////////////////////////////////////////

        // METODI

        //Una soluzione più elegante e modulare sarebbe quella di crare un classe
        //a livello superiori con i parametri di connessione e fargli ereditare dall'Adapeter

        #region AperturaDatabase
        //Apertura della connessione al DB
        public void OpenConnectionToDB()
        {
            // istanziazione dell'oggetto connection
            dbConn = new OleDbConnection();

            // assegnazione della stringa di connessione
            dbConn.ConnectionString = connectionString;

            // apertura della connessione
            dbConn.Open();
        } // end method OpenConnectionToDB
        #endregion

        #region ChiusuraDatabase
        public void CloseConnectionToDB()
        {
            dbConn.Close();
        } // end method CloseConnectionToDB
        #endregion

        #region SelezioneSenzaParametri

        private DataTable ExecuteSelect(string p_sqlSelect)
        {
            // creazione dell'oggetto comando OleDB
            OleDbCommand cmd = new OleDbCommand();

            // associazione della query al comando
            cmd.CommandText = p_sqlSelect;

            // assegnazione della connessione al comando
            cmd.Connection = dbConn;

            // verifica che la connessione sia aperta
            if (dbConn.State == ConnectionState.Closed)
            {
                // e lancia una eccezione in caso contrario
                Exception err = new Exception("Connessione al DB chiusa!");
                throw err;
            }

            // crea un DataReader associato al comando
            OleDbDataReader reader = cmd.ExecuteReader();

            // attraverso il metodo load 
            // i contenuti del datareader 
            // vengono caricati in una datatable
            DataTable dt = new DataTable();
            dt.Load(reader);

            // ritorna la DataTable cosi' ottenuta
            return dt;
        } // end method ExecuteSelect
        #endregion

        #region SelezioneConParametri 
        private DataTable ExecuteSelectWithParam(string p_sqlSelect, string p_key, string p_param)
        {
            // creazione dell'oggetto comando DB
            OleDbCommand cmd = new OleDbCommand();

            // assegnazione della query al comando
            cmd.CommandText = p_sqlSelect;

            // assegnazione della connessione al comando
            cmd.Connection = dbConn;

            // creazione dell'oggetto DbParameter associato alla chiave e al valore
            OleDbParameter par1 = new OleDbParameter(p_key, p_param);

            // aggiunta del parametro al DbCommand
            cmd.Parameters.Add(par1);

            // verifica che la connessione sia aperta
            if (dbConn.State == ConnectionState.Closed)
            {
                // e lancia una eccezione in caso contrario
                Exception err = new Exception("Connessione al DB chiusa!");
                throw err;
            }

            // crea un DataReader associato al comando
            OleDbDataReader reader = cmd.ExecuteReader();

            // attraverso il metodo load 
            // i contenuti del datareader 
            // vengono caricati in una datatable
            DataTable dt = new DataTable();
            dt.Load(reader);

            // ritorna la DataTable cosi' ottenuta
            return dt;
        } // end method ExecuteSelectWithParam
#endregion

        #region CaricamentoParzialeDelRecord
        //Carica solo ID, Nome, Cognome dal record
        public DataTable GetData()
        {
            return this.ExecuteSelect(sql_GetDataDetail);
        }
        #endregion

        #region CaricamentoTotaleDelRecord
        //Carica Tutti i campi di tutti i record presenti nella tabella
        public DataTable GetFullData()
        {
            return this.ExecuteSelect(sql_GetFullData);
        } // end method GetData
        #endregion

        #region CaricamentoParzialeDelRecordPerID
        //Seleziona i dati in base all'id
        public DataTable GetDataByID(int p_id)
        {
            OleDbCommand cmd = new OleDbCommand();
            cmd.CommandText = sql_GetDataByID;
            cmd.Connection = dbConn;

            // verifica che la connessione sia aperta
            if (dbConn.State == ConnectionState.Closed)
            {
                // e lancia una eccezione in caso contrario
                Exception err = new Exception("Connessione al DB chiusa!");
                throw err;
            }

            OleDbParameter par1 = new OleDbParameter("ID", p_id);

            cmd.Parameters.Add(par1);

            // crea un DataReader associato al comando
            OleDbDataReader reader = cmd.ExecuteReader();

            DataTable dt = new DataTable();
            dt.Load(reader);

            return dt;
        } // end method GetDataByID
        #endregion


    } // end class Adapter

} // end namespace CL_Adapter
